# Generated by Django 5.2.11 on 2026-03-14 02:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_qareports", "0022_historicalnote"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="note",
            name="edc_qareports_note_report_model_subj_uniq",
        ),
        migrations.AddField(
            model_name="historicalnote",
            name="visit_code",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AddField(
            model_name="historicalnote",
            name="visit_code_sequence",
            field=models.IntegerField(blank=True, default=0, verbose_name="Sequence"),
        ),
        migrations.AddField(
            model_name="note",
            name="visit_code",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AddField(
            model_name="note",
            name="visit_code_sequence",
            field=models.IntegerField(blank=True, default=0, verbose_name="Sequence"),
        ),
        migrations.AlterField(
            model_name="historicalnote",
            name="status",
            field=models.CharField(
                choices=[
                    ("PENDING", "Data is pending"),
                    ("not_available", "Data not available"),
                    ("closed", "Closed - Final"),
                ],
                default="",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="note",
            name="status",
            field=models.CharField(
                choices=[
                    ("PENDING", "Data is pending"),
                    ("not_available", "Data not available"),
                    ("closed", "Closed - Final"),
                ],
                default="",
                max_length=25,
            ),
        ),
        migrations.AddConstraint(
            model_name="note",
            constraint=models.UniqueConstraint(
                fields=(
                    "report_model",
                    "subject_identifier",
                    "visit_code",
                    "visit_code_sequence",
                ),
                name="edc_qareports_note_report_model_subj_uniq",
            ),
        ),
    ]



from django.apps.config import AppConfig as DjangoAppConfig
from django.core.management.color import color_style

style = color_style()


class AppConfig(DjangoAppConfig):
    name = "edc_visit_schedule"
    verbose_name = "Edc Visit Schedules"
    validate_models = True
    include_in_administration_section = True



# Generated by Django 5.2.11 on 2026-03-13 23:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0135_alter_stock_allocation"),
    ]

    operations = [
        migrations.AddField(
            model_name="stockavailability",
            name="visit_code_sequence",
            field=models.IntegerField(null=True),
        ),
    ]



# Generated by Django 4.0.5 on 2022-07-02 15:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_pharmacy", "0005_alter_rx_managers"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalrxrefill",
            name="roundup_divisible_by",
            field=models.IntegerField(default=1),
        ),
        migrations.AddField(
            model_name="rxrefill",
            name="roundup_divisible_by",
            field=models.IntegerField(default=1),
        ),
    ]



from django.db import models

from edc_action_item.managers import (
    ActionIdentifierModelManager,
    ActionIdentifierSiteManager,
)
from edc_action_item.models import ActionModelMixin
from edc_adverse_event.constants import AESI_ACTION
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierFieldMixin
from edc_model.models import ReportStatusModelMixin
from edc_sites.model_mixins import SiteModelMixin

from .aesi_fields_model_mixin import AesiFieldsModelMixin
from .aesi_methods_model_mixin import AesiMethodsModelMixin


class AesiModelMixin(
    AesiMethodsModelMixin,
    SiteModelMixin,
    ActionModelMixin,
    NonUniqueSubjectIdentifierFieldMixin,
    AesiFieldsModelMixin,
    ReportStatusModelMixin,
    models.Model,
):
    action_name = AESI_ACTION

    objects = ActionIdentifierModelManager()

    on_site = ActionIdentifierSiteManager()

    class Meta(NonUniqueSubjectIdentifierFieldMixin.Meta):
        abstract = True
        verbose_name = "AE of Special Interest Report"
        indexes = (
            *NonUniqueSubjectIdentifierFieldMixin.Meta.indexes,
            *ActionModelMixin.Meta.indexes,
            models.Index(fields=["subject_identifier", "action_identifier", "site", "id"]),
        )



from clinicedc_constants import CLOSED, DATA_ISSUE, NOT_AVAILABLE, PENDING

NOTE_STATUSES = (
    (PENDING, "Data is pending"),
    (NOT_AVAILABLE, "Data not available"),
    (DATA_ISSUE, "Data issue"),
    (CLOSED, "Closed - Final"),
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

from edc_registration import get_registered_subject_model_cls

from ..constants import DO_NOTHING

if TYPE_CHECKING:
    from edc_registration.models import RegisteredSubject
    from edc_visit_tracking.model_mixins import VisitModelMixin as Base

    from ..model_mixins.creates import CreatesMetadataModelMixin
    from .logic import Logic

    class RelatedVisitModel(CreatesMetadataModelMixin, Base):
        pass


class RuleEvaluatorError(Exception):
    pass


class RuleEvaluatorRegisterSubjectError(Exception):
    pass


show_edc_metadata_warnings = getattr(settings, "EDC_METADATA_SHOW_NOVALUEERROR_WARNING", False)


class RuleEvaluator:
    """A class to evaluate a rule.

    Sets `self.result` to REQUIRED, NOT_REQUIRED or None.

    Set as a class attribute on Rule.

    See also RuleGroup and its metaclass.
    """

    def __init__(
        self, logic: Logic = None, related_visit: RelatedVisitModel = None, **kwargs
    ) -> None:
        self._registered_subject: RegisteredSubject | None = None
        self.logic: Logic = logic
        self.result: str | None = None
        self.related_visit = related_visit
        options = dict(
            visit=self.related_visit,
            registered_subject=self.registered_subject,
            **kwargs,
        )
        predicate = self.logic.predicate(**options)
        if predicate:
            if self.logic.consequence != DO_NOTHING:
                self.result = self.logic.consequence
        elif self.logic.alternative != DO_NOTHING:
            self.result = self.logic.alternative

    @property
    def registered_subject_model(self) -> type[RegisteredSubject]:
        return get_registered_subject_model_cls()

    @property
    def registered_subject(self) -> Any:
        """Returns a registered subject model instance or raises."""
        if not self._registered_subject:
            try:
                self._registered_subject = self.registered_subject_model.objects.get(
                    subject_identifier=self.related_visit.subject_identifier
                )
            except ObjectDoesNotExist as e:
                raise RuleEvaluatorRegisterSubjectError(
                    f"Registered subject required for rule {self!r}. "
                    f"subject_identifier='{self.related_visit.subject_identifier}'. "
                    f"Got {e}."
                ) from e
        return self._registered_subject



# Generated by Django 3.2.13 on 2022-08-25 23:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_protocol_incident",
            "0008_alter_historicalprotocoldeviationviolation_action_identifier_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalprotocoldeviationviolation",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name="protocoldeviationviolation",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, null=True, unique=True),
        ),
    ]



# Generated by Django 5.2.11 on 2026-03-17 01:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_qareports",
            "0023_remove_note_edc_qareports_note_report_model_subj_uniq_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalnote",
            name="status",
            field=models.CharField(
                choices=[
                    ("PENDING", "Data is pending"),
                    ("not_available", "Data not available"),
                    ("DATA_ISSUE", "Data issue"),
                    ("closed", "Closed - Final"),
                ],
                default="",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="note",
            name="status",
            field=models.CharField(
                choices=[
                    ("PENDING", "Data is pending"),
                    ("not_available", "Data not available"),
                    ("DATA_ISSUE", "Data issue"),
                    ("closed", "Closed - Final"),
                ],
                default="",
                max_length=25,
            ),
        ),
    ]



# Generated by Django 5.2.11 on 2026-03-05 18:39

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0134_alter_storagebinproxy_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="stock",
            name="allocation",
            field=models.OneToOneField(
                blank=True,
                help_text="Subject allocation",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="edc_pharmacy.allocation",
            ),
        ),
    ]



from importlib.metadata import version
from pathlib import Path

from django.apps import apps as django_apps
from django.conf import settings
from django.utils import timezone

from .form_describer import FormDescriber
from .markdown_writer import MarkdownWriter


class FormsReference:
    describer_cls = FormDescriber
    markdown_writer_cls = MarkdownWriter
    anchor_prefix = "user-content"
    h1 = "#"
    h2 = "##"
    h3 = "###"
    h4 = "####"

    def __init__(
        self,
        visit_schedules=None,
        admin_site=None,
        include_hidden_fields=None,
        title=None,
        add_per_form_timestamp: bool | None = None,
    ):
        self.toc = []
        self.title = title or "Forms Reference"
        self._anchors = []
        self._markdown = []
        self.visit_schedules = visit_schedules
        self.admin_site = admin_site
        self.include_hidden_fields = include_hidden_fields
        self.plans = {}
        self.add_per_form_timestamp = (
            True if add_per_form_timestamp is None else add_per_form_timestamp
        )
        self.timestamp = timezone.now().strftime("%Y-%m-%d %H:%M")
        for visit_schedule in self.visit_schedules:
            self.plans.update({visit_schedule.name: {}})
            for schedule in visit_schedule.schedules.values():
                for visit_code, visit in schedule.visits.items():
                    crfs = [c.model for c in visit.crfs]
                    requisitions = [r.panel.name for r in visit.requisitions]
                    self.plans[visit_schedule.name].update(
                        {visit_code: {"crfs": crfs, "requisitions": requisitions}}
                    )

    def to_file(
        self,
        path: Path | str | None = None,
        overwrite: bool | None = None,
        pad: int | None = None,
    ):
        pad = pad if pad is not None else 2
        markdown_writer = self.markdown_writer_cls(path=path, overwrite=overwrite)
        markdown_writer.to_file(markdown=self.markdown, pad=pad)

    def insert_toc(self, toc=None, markdown=None):
        toc.reverse()
        markdown.insert(0, "\n")
        for line in toc:
            markdown.insert(0, line)
        markdown.insert(0, f"{self.h2} Table of contents\n")
        return markdown

    def get_anchor(self, anchor=None):
        index = 0
        anchor_orig = anchor
        while True:
            if anchor not in self._anchors:
                self._anchors.append(anchor)
                break
            index += 1
            anchor = anchor_orig + f"-{index}"
        return anchor

    @property
    def markdown(self):
        if not self._markdown:
            markdown = []
            toc = []
            for plan in self.plans.values():
                for visit_code, documents in plan.items():
                    markdown.append(f"{self.h3} {visit_code}\n")
                    toc.append(
                        f'\n<a href="#{self.anchor_prefix}-{visit_code.lower()}">'
                        f"**{visit_code}.**</a>"
                    )
                    for index, model in enumerate(documents.get("crfs")):
                        model_cls = django_apps.get_model(model)
                        admin_cls = self.admin_site._registry.get(model_cls)
                        if admin_cls:
                            describer = self.describer_cls(
                                admin_cls=admin_cls,
                                include_hidden_fields=self.include_hidden_fields,
                                visit_code=visit_code,
                                level=self.h4,
                                anchor_prefix=self.anchor_prefix,
                                add_timestamp=self.add_per_form_timestamp,
                            )
                            describer.markdown.append("\n")
                            anchor = f"{self.get_anchor(describer.anchor)}"
                            toc.append(
                                f'{index + 1}. <a href="#{anchor}">{describer.verbose_name}</a>'
                            )
                            markdown.extend(describer.markdown)
                    markdown.append(f"{self.h4} Requisitions\n")
                    markdown.extend(
                        [f"* {panel_name}\n" for panel_name in documents.get("requisitions")]
                    )
            markdown = self.insert_toc(toc, markdown)
            markdown.insert(0, f"{self.h1} {self.title}")
            markdown.append(
                f"\n\n* Version v{version(settings.APP_NAME)} "
                f"* Rendered on {self.timestamp}*\n"
            )
            self._markdown = markdown
        return self._markdown



from edc_auth.constants import (
    AUDITOR_ROLE,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    NURSE_ROLE,
)
from edc_auth.site_auths import site_auths

from .auth_objects import APPOINTMENT, APPOINTMENT_EXPORT, APPOINTMENT_VIEW, codenames


def update_site_auths():
    site_auths.add_group(*codenames, name=APPOINTMENT_VIEW, view_only=True)

    site_auths.add_group(*codenames, name=APPOINTMENT)

    site_auths.add_group(
        "edc_appointment.export_appointment",
        name=APPOINTMENT_EXPORT,
    )

    site_auths.update_role(APPOINTMENT, name=CLINICIAN_ROLE)
    site_auths.update_role(APPOINTMENT, name=NURSE_ROLE)
    site_auths.update_role(APPOINTMENT_VIEW, name=AUDITOR_ROLE)
    site_auths.update_role(APPOINTMENT, name=CLINICIAN_SUPER_ROLE)


update_site_auths()



import sys

from django.apps import apps as django_apps
from django.core.management import BaseCommand

from edc_qareports.model_mixins import QaReportModelMixin


class Command(BaseCommand):
    help = (
        "Safely recreate the database view for models declared "
        "with `django_db_views.DBView` to use a local account "
        "as definer."
    )

    def handle(self, *args, **options):
        for model_cls in django_apps.get_models():
            if issubclass(model_cls, (QaReportModelMixin,)):
                sys.stdout.write(f"{model_cls._meta.db_table}\n")
                try:
                    model_cls.recreate_db_view()
                except AttributeError as e:
                    sys.stdout.write(f"{e}\n")
                except TypeError as e:
                    sys.stdout.write(f"{e}\n")
